#
# SIP Conference Bot
#
# Copyright (C) 2008-2009 Teluu Inc. (http://www.teluu.com)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
#
import pjsua as pj
import string
import sys

CFG_FILE = "config"

INFO = 1
TRACE = 2
	
# Call callback. This would just forward the event to the Member class
class CallCb(pj.CallCallback):
	def __init__(self, member, call=None):
		pj.CallCallback.__init__(self, call)
		self.member = member
	
	def on_state(self):
		self.member.on_call_state(self.call)
	
	def on_media_state(self):
		self.member.on_call_media_state(self.call)

	def on_dtmf_digit(self, digits):
		self.member.on_call_dtmf_digit(self.call, digits)

	def on_transfer_request(self, dst, code):
		return self.member.on_call_transfer_request(self.call, dst, code)
	
	def on_transfer_status(self, code, reason, final, cont):
		return self.member.on_call_transfer_status(self.call, code, reason, final, cont)

	def on_replace_request(self, code, reason):
		return self.member.on_call_replace_request(self.call, code, reason)
		
	def on_replaced(self, new_call):
		self.member.on_call_replaced(self.call, new_call)

	def on_typing(self, is_typing):
		self.member.on_typing(is_typing, call=self.call)

	def on_pager(self, mime_type, body):
		self.member.on_pager(mime_type, body, call=self.call)
	
	def on_pager_status(self, body, im_id, code, reason):
		self.member.on_pager_status(body, im_id, code, reason, call=self.call)

# Buddy callback. This would just forward the event to Member class
class BuddyCb(pj.BuddyCallback):
	def __init__(self, member, buddy=None):
		pj.BuddyCallback.__init__(self, buddy)
		self.member = member
	
	def on_pager(self, mime_type, body):
		self.member.on_pager(mime_type, body, buddy=self.buddy)

	def on_pager_status(self, body, im_id, code, reason):
		self.member.on_pager_status(body, im_id, code, reason, buddy=self.buddy)

	def on_state(self):
		self.member.on_pres_state(self.buddy)

	def on_typing(self, is_typing):
		self.member.on_typing(is_typing, buddy=self.buddy)




##############################################################################
#
#
# This class represents individual room member (either/both chat and voice conf)
#
#
class Member:
	def __init__(self, bot, uri):
		self.uri = uri
		self.bot = bot
		self.call = None
		self.buddy = None
		self.bi = pj.BuddyInfo()
		self.in_chat = False
		self.in_voice = False
		self.im_error = False
		self.html = False
	
	def __str__(self):
		str = string.ljust(self.uri, 30) + " -- " 
		if self.buddy:
			bi = self.buddy.info()
			str = str + bi.online_text
		else:
			str = str + "Offline"
		str = str + " ["
		if (self.in_voice):
			str = str + " voice"
		if (self.in_chat):
			str = str + " chat"
			if (self.html):
				str = str + " html"
			else:
				str = str + " plain"

		if (self.im_error):
			str = str + " im_error"
		str = str + "]"
		return str
		
	def join_call(self, call):
		if self.call:
			self.call.hangup(603, "You have been disconnected for making another call")
		self.call = call
		call.set_callback(CallCb(self, call))
		msg = "%(uri)s is attempting to join the voice conference" % \
			  {'uri': self.uri}
		self.bot.DEBUG(msg + "\n", INFO)
		self.bot.broadcast_pager(None, msg)
	
	def join_chat(self):
		if not self.buddy:
			self.bot.DEBUG(self.uri + " joining chatroom...\n", INFO)
			self.buddy = self.bot.acc.add_buddy(self.uri)
			self.buddy.set_callback(BuddyCb(self, self.buddy))
			self.buddy.subscribe()
		else:
			self.bot.DEBUG(self.uri + " already in chatroom, resubscribing..\n", INFO)
			self.buddy.subscribe()

	def send_pager(self, body, mime="text/plain"):
		self.bot.DEBUG("send_pager() to " + self.uri)
		if self.in_chat and not self.im_error and self.buddy:
			if self.html:
				#This will make us receive html!
				#mime = "text/html"
				body = body.replace("<", "&lt;")
				body = body.replace(">", "&gt;")
				body = body.replace('"', "&quot;")
				body = body.replace("\n", "<BR>\n")
			self.buddy.send_pager(body, content_type=mime)
			self.bot.DEBUG("..sent\n")
		else:
			self.bot.DEBUG("..not sent!\n")
		
	def on_call_state(self, call):
		ci = call.info()
		if ci.state==pj.CallState.DISCONNECTED:
			if self.in_voice:
				msg = "%(uri)s has left the voice conference (%(1)d/%(2)s)" % \
					  {'uri': self.uri, '1': ci.last_code, '2': ci.last_reason}
				self.bot.DEBUG(msg + "\n", INFO)
				self.bot.broadcast_pager(None, msg)
			self.in_voice = False
			self.call = None
			self.bot.on_member_left(self)
		elif ci.state==pj.CallState.CONFIRMED:
			msg = "%(uri)s has joined the voice conference" % \
				  {'uri': self.uri}
			self.bot.DEBUG(msg + "\n", INFO)
			self.bot.broadcast_pager(None, msg)
	
	def on_call_media_state(self, call):
		self.bot.DEBUG("Member.on_call_media_state\n")
		ci = call.info()
		if ci.conf_slot!=-1:
			if not self.in_voice:
				msg = self.uri + " call media is active"
				self.bot.broadcast_pager(None, msg)
			self.in_voice = True
			self.bot.add_to_voice_conf(self)
		else:
			if self.in_voice:
				msg = self.uri + " call media is inactive"
				self.bot.broadcast_pager(None, msg)
			self.in_voice = False

	def on_call_dtmf_digit(self, call, digits):
		msg = "%(uri)s sent DTMF digits %(dig)s" % \
			  {'uri': self.uri, 'dig': digits}
		self.bot.broadcast_pager(None, msg)

	def on_call_transfer_request(self, call, dst, code):
		msg = "%(uri)s is transferring the call to %(dst)s" % \
			  {'uri': self.uri, 'dst': dst}
		self.bot.broadcast_pager(None, msg)
		return 202
	
	def on_call_transfer_status(self, call, code, reason, final, cont):
		msg = "%(uri)s call transfer status is %(code)d/%(res)s" % \
			  {'uri': self.uri, 'code': code, 'res': reason}
		self.bot.broadcast_pager(None, msg)
		return True

	def on_call_replace_request(self, call, code, reason):
		msg = "%(uri)s is requesting call replace" % \
			  {'uri': self.uri}
		self.bot.broadcast_pager(None, msg)
		return (code, reason)
		
	def on_call_replaced(self, call, new_call):
		msg = "%(uri)s call is replaced" % \
			  {'uri': self.uri}
		self.bot.broadcast_pager(None, msg)

	def on_pres_state(self, buddy):
		old_bi = self.bi
		self.bi = buddy.info()
		msg = "%(uri)s status is %(st)s" % \
		  {'uri': self.uri, 'st': self.bi.online_text}
		self.bot.DEBUG(msg + "\n", INFO)
		self.bot.broadcast_pager(self, msg)

		if self.bi.sub_state==pj.SubscriptionState.ACTIVE:
			if not self.in_chat:
				self.in_chat = True
				buddy.send_pager("Welcome to chatroom")
				self.bot.broadcast_pager(self, self.uri + " has joined the chat room")
			else:
				self.in_chat = True
		elif self.bi.sub_state==pj.SubscriptionState.NULL or \
			 self.bi.sub_state==pj.SubscriptionState.TERMINATED or \
			 self.bi.sub_state==pj.SubscriptionState.UNKNOWN:
			self.buddy.delete()
			self.buddy = None
			if self.in_chat:
				self.in_chat = False
				self.bot.broadcast_pager(self, self.uri + " has left the chat room")
			else:
				self.in_chat = False
			self.bot.on_member_left(self)
		
	def on_typing(self, is_typing, call=None, buddy=None):
		if is_typing:
			msg = self.uri + " is typing..."
		else:
			msg = self.uri + " has stopped typing"
		self.bot.broadcast_pager(self, msg)

	def on_pager(self, mime_type, body, call=None, buddy=None):
		if not self.bot.handle_cmd(self, None, body):
			msg = self.uri + ": " + body
			self.bot.broadcast_pager(self, msg, mime_type)
	
	def on_pager_status(self, body, im_id, code, reason, call=None, buddy=None):
		self.im_error = (code/100 != 2)



##############################################################################
#
#
# The Bot instance (singleton)
#
#
class Bot(pj.AccountCallback):
	def __init__(self):
		pj.AccountCallback.__init__(self, None)
		self.lib = pj.Lib()
		self.acc = None
		self.calls = []
		self.members = {}
		self.cfg = None

	def DEBUG(self, msg, level=TRACE):
		print msg,
		
	def helpstring(self):
		return """
--h[elp]            Display this help screen
--j[oin]            Join the chat room
--html on|off       Set to receive HTML or plain text

Participant commands:
--s[how]            Show confbot settings
--leave             Leave the chatroom
--l[ist]            List all members

Admin commands:
--a[dmin] <CMD> Where <CMD> are:
    list            List the admins
    add <URI>       Add URI as admin
    del <URI>       Remove URI as admin
    rr              Reregister account to server
    call <URI>      Make call to the URI and add to voice conf
    dc <URI>        Disconnect call to URI
    hold <URI>      Hold call with that URI
    update <URI>    Send UPDATE to call with that URI
    reinvite <URI>  Send re-INVITE to call with that URI
"""

	def listmembers(self):
		msg = ""
		for uri, m in self.members.iteritems():
			msg = msg + str(m) + "\n"
		return msg
	
	def showsettings(self):
		ai = self.acc.info()
		msg = """
ConfBot status and settings:
  URI:        %(uri)s
  Status:     %(pres)s
  Reg Status: %(reg_st)d
  Reg Reason: %(reg_res)s
""" % {'uri': ai.uri, 'pres': ai.online_text, \
	   'reg_st': ai.reg_status, 'reg_res': ai.reg_reason}
		return msg
  
	def main(self, cfg_file):
		try:
			cfg = self.cfg = __import__(cfg_file)
			
			self.lib.init(ua_cfg=cfg.ua_cfg, log_cfg=cfg.log_cfg, media_cfg=cfg.media_cfg)
			self.lib.set_null_snd_dev()
			
			transport = None
			if cfg.udp_cfg:
				transport = self.lib.create_transport(pj.TransportType.UDP, cfg.udp_cfg)
			if cfg.tcp_cfg:
				t = self.lib.create_transport(pj.TransportType.TCP, cfg.tcp_cfg)
				if not transport:
					transport = t
				
			self.lib.start()
			
			if cfg.acc_cfg:
				self.DEBUG("Creating account %(uri)s..\n" % {'uri': cfg.acc_cfg.id}, INFO)
				self.acc = self.lib.create_account(cfg.acc_cfg, cb=self)
			else:
				self.DEBUG("Creating account for %(t)s..\n" % \
							{'t': transport.info().description}, INFO)
				self.acc = self.lib.create_account_for_transport(transport, cb=self)
			
			self.acc.set_basic_status(True)
			
			# Wait for ENTER before quitting
			print "Press q to quit or --help/--h for help"
			while True:
				input = sys.stdin.readline().strip(" \t\r\n")
				if not self.handle_cmd(None, None, input):
					if input=="q":
						break
			
			self.lib.destroy()
			self.lib = None

		except pj.Error, e:
			print "Exception: " + str(e)
			if self.lib:
				self.lib.destroy()
				self.lib = None
	
	def broadcast_pager(self, exclude_member, body, mime_type="text/plain"):
		self.DEBUG("Broadcast: " + body + "\n")
		for uri, m in self.members.iteritems():
			if m != exclude_member:
				m.send_pager(body, mime_type)

	def add_to_voice_conf(self, member):
		if not member.call:
			return
		src_ci = member.call.info()
		self.DEBUG("bot.add_to_voice_conf\n")
		for uri, m in self.members.iteritems():
			if m==member:
				continue
			if not m.call:
				continue
			dst_ci = m.call.info()
			if dst_ci.media_state==pj.MediaState.ACTIVE and dst_ci.conf_slot!=-1:
				self.lib.conf_connect(src_ci.conf_slot, dst_ci.conf_slot)
				self.lib.conf_connect(dst_ci.conf_slot, src_ci.conf_slot)
	
	def on_member_left(self, member):
		if not member.call and not member.buddy:
			del self.members[member.uri]
			del member
			
	def handle_admin_cmd(self, member, body):
		if member and self.cfg.admins and not member.uri in self.cfg.admins:
			member.send_pager("You are not admin")
			return
		args = body.split()
		msg = ""

		if len(args)==1:
			args.append(" ")

		if args[1]=="list":
			if not self.cfg.admins:
				msg = "Everyone is admin!"
			else:
				msg = str(self.cfg.admins)
		elif args[1]=="add":
			if len(args)!=3:
				msg = "Usage: add <URI>"
			else:
				self.cfg.admins.append(args[2])
				msg = args[2] + " added as admin"
		elif args[1]=="del":
			if len(args)!=3:
				msg = "Usage: del <URI>"
			elif args[2] not in self.cfg.admins:
				msg = args[2] + " is not admin"
			else:
				self.cfg.admins.remove(args[2])
				msg = args[2] + " has been removed from admins"
		elif args[1]=="rr":
			msg = "Reregistering.."
			self.acc.set_registration(True)
		elif args[1]=="call":
			if len(args)!=3:
				msg = "Usage: call <URI>"
			else:
				uri = args[2]
				try:
					call = self.acc.make_call(uri)
				except pj.Error, e:
					msg = "Error: " + str(e)
					call = None

				if call:
					if not uri in self.members:
						m = Member(self, uri)
						self.members[m.uri] = m
					else:
						m = self.members[uri]
					msg = "Adding " + m.uri + " to voice conference.."
					m.join_call(call)
		elif args[1]=="dc" or args[1]=="hold" or args[1]=="update" or args[1]=="reinvite":
			if len(args)!=3:
				msg = "Usage: " + args[1] + " <URI>"
			else:
				uri = args[2]
				if not uri in self.members:
					msg = "Member not found/URI doesn't match (note: case matters!)"
				else:
					m = self.members[uri]
					if m.call:
						if args[1]=="dc":
							msg = "Disconnecting.."
							m.call.hangup(603, "You're disconnected by admin")
						elif args[1]=="hold":
							msg = "Holding the call"
							m.call.hold()
						elif args[1]=="update":
							msg = "Sending UPDATE"
							m.call.update()
						elif args[1]=="reinvite":
							msg = "Sending re-INVITE"
							m.call.reinvite()
					else:
						msg = "He is not in call"
		else:
			msg = "Unknown admin command " + body

		#print "msg is '%(msg)s'" % {'msg': msg}

		if True:
			if member:
				member.send_pager(msg)
			else:
				print msg

	def handle_cmd(self, member, from_uri, body):
		body = body.strip(" \t\r\n")
		msg = ""
		handled = True
		if body=="--l" or body=="--list":
			msg = self.listmembers()
			if msg=="":
				msg = "Nobody is here"
		elif body[0:3]=="--s":
			msg = self.showsettings()
		elif body[0:6]=="--html" and member:
			if body[8:11]=="off":
				member.html = False
			else:
				member.html = True
		elif body=="--h" or body=="--help":
			msg = self.helpstring()
		elif body=="--leave":
			if not member or not member.buddy:
				msg = "You are not in chatroom"
			else:
				member.buddy.unsubscribe()
		elif body[0:3]=="--j":
			if not from_uri in self.members:
				m = Member(self, from_uri)
				self.members[m.uri] = m
				self.DEBUG("Adding " + m.uri + " to chatroom\n")
				m.join_chat()
			else:
				m = self.members[from_uri]
				self.DEBUG("Adding " + m.uri + " to chatroom\n")
				m.join_chat()
		elif body[0:3]=="--a":
			self.handle_admin_cmd(member, body)
			handled = True
		else:
			handled = False

		if msg:
			if member:
				member.send_pager(msg)
			elif from_uri:
				self.acc.send_pager(from_uri, msg);
			else:
				print msg
		return handled
	
	def on_incoming_call(self, call):
		self.DEBUG("on_incoming_call from %(uri)s\n" % {'uri': call.info().remote_uri}, INFO)
		ci = call.info()
		if not ci.remote_uri in self.members:
			m = Member(self, ci.remote_uri)
			self.members[m.uri] = m
			m.join_call(call)
		else:
			m = self.members[ci.remote_uri]
			m.join_call(call)
		call.answer(200)
	
	def on_incoming_subscribe(self, buddy, from_uri, contact_uri, pres_obj):
		self.DEBUG("on_incoming_subscribe from %(uri)s\n" % from_uri, INFO)
		return (200, 'OK')

	def on_reg_state(self):
		ai = self.acc.info()
		self.DEBUG("Registration state: %(code)d/%(reason)s\n" % \
					{'code': ai.reg_status, 'reason': ai.reg_reason}, INFO)
		if ai.reg_status/100==2 and ai.reg_expires > 0:
			self.acc.set_basic_status(True)

	def on_pager(self, from_uri, contact, mime_type, body):
		body = body.strip(" \t\r\n")
		if not self.handle_cmd(None, from_uri, body):
			self.acc.send_pager(from_uri, "You have not joined the chat room. Type '--join' to join or '--help' for the help")

	def on_pager_status(self, to_uri, body, im_id, code, reason):
		pass

	def on_typing(self, from_uri, contact, is_typing):
		pass




##############################################################################
#
#
# main()
#
#
if __name__ == "__main__":
	bot = Bot()
	bot.main(CFG_FILE)




import inc_sip as sip
import inc_sdp as sdp

# Incoming MESSAGE without body is now accepted
#
complete_msg = \
"""MESSAGE sip:localhost SIP/2.0
Via: SIP/2.0/UDP 192.168.0.14:5060;rport;branch=z9hG4bKPj9db9
Max-Forwards: 70
From: <sip:192.168.0.14>;tag=08cd5bfc2d8a4fddb1f5e59c6961d298
To: <sip:localhost>
Call-ID: 3373d9eb32aa458db7e69c7ea51e0bd7
CSeq: 23809 MESSAGE
Contact: <sip:192.168.0.14:5060>
User-Agent: PJSUA v0.8.0-trunk/win32
Content-Type: text/plain
Content-Length: 50
"""


sendto_cfg = sip.SendtoCfg( "empty MESSAGE", "--null-audio --auto-answer 200", 
			    "", 200, complete_msg=complete_msg)




import py_pjsua

status = py_pjsua.create()
print "py status " + `status`


#
# Create configuration objects
#
ua_cfg = py_pjsua.config_default()
log_cfg = py_pjsua.logging_config_default()
media_cfg = py_pjsua.media_config_default()

#
# Logging callback.
#
def logging_cb1(level, str, len):
    print str,


#
# Configure logging
#
log_cfg.cb = logging_cb1
log_cfg.console_level = 4

#
# Initialize pjsua!
#
status = py_pjsua.init(ua_cfg, log_cfg, media_cfg);
print "py status after initialization :" + `status`


#
# Start pjsua!
#
status = py_pjsua.start()
if status != 0:
    exit(1)


message = py_pjsua.msg_data_init()

print "identitas object message data :" + `message`

sipaddr = 'sip:167.205.34.99'
print "checking sip address [%s] : %d" % (sipaddr, py_pjsua.verify_sip_url(sipaddr))

sipaddr = '167.205.34.99'
print "checking invalid sip address [%s] : %d" % (sipaddr, py_pjsua.verify_sip_url(sipaddr))

object = py_pjsua.get_pjsip_endpt()
print "identitas Endpoint :" + `object` + ""

mediaend = py_pjsua.get_pjmedia_endpt()
print "identitas Media Endpoint :" + `mediaend` + ""

pool = py_pjsua.get_pool_factory()
print "identitas pool factory :" + `pool` + ""

status = py_pjsua.handle_events(3000)
print "py status after 3 second of blocking wait :" + `status`



# end of new testrun

#

# lib transport
stunc = py_pjsua.stun_config_default();


tc = py_pjsua.transport_config_default();


py_pjsua.normalize_stun_config(stunc);


status, id = py_pjsua.transport_create(1, tc);
print "py transport create status " + `status`

ti = py_pjsua.Transport_Info();
ti = py_pjsua.transport_get_info(id)
print "py transport get info status " + `status`

status = py_pjsua.transport_set_enable(id,1)
print "py transport set enable status " + `status`
if status != 0 :
	py_pjsua.perror("py_pjsua","set enable",status)


status = py_pjsua.transport_close(id,1)
print "py transport close status " + `status`
if status != 0 :
	py_pjsua.perror("py_pjsua","close",status)

# end of lib transport

# lib account 

accfg = py_pjsua.acc_config_default()
status, accid = py_pjsua.acc_add(accfg, 1)
print "py acc add status " + `status`
if status != 0 :
	py_pjsua.perror("py_pjsua","add acc",status)
count = py_pjsua.acc_get_count()
print "acc count " + `count`

accid = py_pjsua.acc_get_default()

print "acc id default " + `accid`

# end of lib account

#lib buddy

bcfg = py_pjsua.Buddy_Config()
status, id = py_pjsua.buddy_add(bcfg)
acc_id = id
print "py buddy add status " + `status` + " id " + `id`
bool = py_pjsua.buddy_is_valid(id)
print "py buddy is valid " + `bool`
count = py_pjsua.get_buddy_count()
print "buddy count " + `count`
binfo = py_pjsua.buddy_get_info(id)
ids = py_pjsua.enum_buddies()
status = py_pjsua.buddy_del(id)
print "py buddy del status " + `status`
status = py_pjsua.buddy_subscribe_pres(id, 1)
print "py buddy subscribe pres status " + `status`
py_pjsua.pres_dump(1)
status = py_pjsua.im_send(accid, "fahris@divusi.com", "", "hallo", message, 0)
print "py im send status " + `status`
status = py_pjsua.im_typing(accid, "fahris@divusi.com", 1, message)
print "py im typing status " + `status`
#print "binfo " + `binfo`

#end of lib buddy

#lib media
count = py_pjsua.conf_get_max_ports()
print "py media conf get max ports " + `count`
count = py_pjsua.conf_get_active_ports()
print "py media conf get active ports " + `count`
ids = py_pjsua.enum_conf_ports()
for id in ids:
	print "py media conf ports " + `id`
	cp_info = py_pjsua.conf_get_port_info(id)
	print "port info name " + cp_info.name
pool = py_pjsua.PJ_Pool()
port = py_pjsua.PJMedia_Port()
status, id = py_pjsua.conf_add_port(pool,port)
print "py media conf add port status " + `status` + " id " + `id`
if status != 0 :
	py_pjsua.perror("py_pjsua","add port",status)
status = py_pjsua.conf_remove_port(id)
print "py media conf remove port status " + `status`
if status != 0 :
	py_pjsua.perror("py_pjsua","remove port",status)
status = py_pjsua.conf_connect(id, id)
print "py media conf connect status " + `status`
if status != 0 :
	py_pjsua.perror("py_pjsua","connect",status)
status = py_pjsua.conf_disconnect(id, id)
print "py media conf disconnect status " + `status`
if status != 0 :
	py_pjsua.perror("py_pjsua","disconnect",status)
status, id = py_pjsua.player_create("test.wav", 0)
print "py media player create status " + `status` + " id " + `id`
if status != 0 :
	py_pjsua.perror("py_pjsua","player create",status)
c_id = py_pjsua.player_get_conf_port(id)
print "py media player get conf port id " + `c_id`
status = py_pjsua.player_set_pos(id, 10)
if status != 0 :
	py_pjsua.perror("py_pjsua","player set pos",status)
status = py_pjsua.player_destroy(id)
if status != 0 :
	py_pjsua.perror("py_pjsua","player destroy",status)
status, id = py_pjsua.recorder_create("rec.wav", 0, "None", 1000, 0)
print "py media recorder create status " + `status` + " id " + `id`
if status != 0 :
	py_pjsua.perror("py_pjsua","recorder create",status)
status = py_pjsua.recorder_get_conf_port(id)
print "py media recorder get conf port status " + `status`
if status != 0 :
	py_pjsua.perror("py_pjsua","recorder get conf port",status)
status = py_pjsua.recorder_destroy(id)
print "py media recorder destroy status " + `status`
if status != 0 :
	py_pjsua.perror("py_pjsua","recorder destroy",status)
#cdev, pdev = py_pjsua.get_snd_dev()
#print "py media get snd dev capture dev " + `cdev` + " playback dev " + `pdev`
status = py_pjsua.set_snd_dev(0,1)
print "py media set snd dev status " + `status`
if status != 0 :
	py_pjsua.perror("py_pjsua","set snd dev",status)
status = py_pjsua.set_null_snd_dev()
print "py media set null snd dev status " + `status`
if status != 0 :
	py_pjsua.perror("py_pjsua","set null snd dev",status)
port = py_pjsua.set_no_snd_dev()
status = py_pjsua.set_ec(0,0)
print "py media set ec status " + `status`
if status != 0 :
	py_pjsua.perror("py_pjsua","set ec",status)
tail = py_pjsua.get_ec_tail()
print "py media get ec tail " + `tail`
infos = py_pjsua.enum_codecs()
for info in infos:
	print "py media enum codecs " + `info`
status = py_pjsua.codec_set_priority("coba", 0)
print "py media codec set priority " + `status`
if status != 0 :
	py_pjsua.perror("py_pjsua","codec set priority",status)
c_param = py_pjsua.codec_get_param("coba")
status = py_pjsua.codec_set_param("coba", c_param)
print "py media codec set param " + `status`
if status != 0 :
	py_pjsua.perror("py_pjsua","codec set param",status)
	
#end of lib media

#lib call

count = py_pjsua.call_get_max_count()
print "py call get max count " + `count`
count = py_pjsua.call_get_count()
print "py call get count " + `count`
ids = py_pjsua.enum_calls()
for id in ids:
	print "py enum calls id " + `id`
msg_data = py_pjsua.Msg_Data()
status, id = py_pjsua.call_make_call(-1, "sip:bulukucing1@iptel.org", 0, 0, msg_data)
print "py call make call " + `status` + " id " + `id`
if status != 0 :
	py_pjsua.perror("py_pjsua","call make call",status)
bool = py_pjsua.call_is_active(id)
print "py call is active " + `bool`
bool = py_pjsua.call_has_media(id)
print "py call has media " + `bool`
cp_id = py_pjsua.call_get_conf_port(id)
print "py call get conf port " + `cp_id`
info = py_pjsua.call_get_info(id)
if info != None :
	print "py info id " + `info.id`
status = py_pjsua.call_set_user_data(id, 0)
print "py call set user data status " + `status`
if status != 0 :
	py_pjsua.perror("py_pjsua","set user data",status)
user_data = py_pjsua.call_get_user_data(id)
print "py call get user data " + `user_data`



#end of lib call

py_pjsua.perror("saya","hallo",70006)

status = py_pjsua.destroy()
print "py status " + `status`





from distutils.core import setup, Extension
import os
import sys

# Fill in pj_inc_dirs
pj_inc_dirs = []
f = os.popen("make -f helper.mak inc_dir")
for line in f:
	pj_inc_dirs.append(line.rstrip("\r\n"))
f.close()

# Fill in pj_lib_dirs
pj_lib_dirs = []
f = os.popen("make -f helper.mak lib_dir")
for line in f:
	pj_lib_dirs.append(line.rstrip("\r\n"))
f.close()

# Fill in pj_libs
pj_libs = []
f = os.popen("make -f helper.mak libs")
for line in f:
	pj_libs.append(line.rstrip("\r\n"))
f.close()

# Mac OS X depedencies
if sys.platform == 'darwin':
	extra_link_args = ["-framework", "CoreFoundation", 
			   "-framework", "AudioToolbox"]
else:
	extra_link_args = []


setup(name="py_pjsua", version="0.8",
	ext_modules = [
		Extension("py_pjsua", 
			  ["py_pjsua.c"], 
			  define_macros=[('PJ_AUTOCONF', '1'),],
			  include_dirs=pj_inc_dirs, 
			  library_dirs=pj_lib_dirs, 
			  libraries=pj_libs,
			  extra_link_args=extra_link_args),
	])




import inc_sip as sip
import inc_sdp as sdp

# Video uses dynamic payload type
sdp = \
"""
v=0
o=- 0 0 IN IP4 127.0.0.1
s=-
c=IN IP4 127.0.0.1
t=0 0
m=audio 5000 RTP/AVP 0
m=video 4000 RTP/AVP 100
a=rtpmap:100 myvideo/80000
"""

pjsua_args = "--null-audio --auto-answer 200"
extra_headers = ""
include = ["Content-Type: application/sdp",	# response must include SDP
	   "m=audio [1-9]+[0-9]* RTP/AVP[\\s\\S]+m=video 0 RTP/AVP"
	   ]
exclude = []

sendto_cfg = sip.SendtoCfg("Mixed audio and video", pjsua_args, sdp, 200,
			   extra_headers=extra_headers,
			   resp_inc=include, resp_exc=exclude) 




import os
import sys
import time
import re
import shutil
import platform

PYTHON = os.path.basename(sys.executable)

# Usage:
#  runall.py [test-to-resume]


# Initialize test list
tests = []

# Excluded tests (because they fail?)
excluded_tests = [
    "svn",
    "pyc",
    "scripts-call/150_srtp_2_1",                     # SRTP optional 'cannot' call SRTP mandatory
    "scripts-call/150_srtp_2_3.py",                  # disabled because #1267 wontfix
    "scripts-call/301_ice_public_a.py",              # Unreliable, proxy returns 408 sometimes
    "scripts-call/301_ice_public_b.py",              # Doesn't work because OpenSER modifies SDP
    "scripts-pres/200_publish.py",                   # Ok from cmdline, error from runall.py
    "scripts-media-playrec/100_resample_lf_8_11.py", # related to clock-rate 11 kHz problem
    "scripts-media-playrec/100_resample_lf_8_22.py", # related to clock-rate 22 kHz problem
    "scripts-media-playrec/100_resample_lf_11",      # related to clock-rate 11 kHz problem
    "pesq",                                          # temporarily disabling all pesq related test due to unreliability
    # TODO check all tests below for false negatives
    "call_305_ice_comp_1_2",
    "scripts-sendto/155_err_sdp_bad_syntax",
    "transfer-attended",
    "uac-inv-and-ack-without-sdp",
    "uac-subscribe",
    "uac-ticket-1148",
    "uas-422-then-200-bad-se",
    "uas-answer-180-multiple-fmts-support-update",
    "uas-inv-answered-with-srtp",
    "uas-mwi-0",
    "uas-mwi",
    "uas-register-ip-change-port-only",
    "uas-register-ip-change",
    "uas-timer-update"
]

# Exclude scripts-sipp/uac-reinvite-bad-via-branch on MacOS due to unreliable result
if platform.system()=='Darwin':
    excluded_tests.append("scripts-sipp/uac-reinvite-bad-via-branch")


# Add basic tests
for f in os.listdir("scripts-run"):
    tests.append("mod_run.py scripts-run/" + f)

# Add basic call tests
for f in os.listdir("scripts-call"):
    tests.append("mod_call.py scripts-call/" + f)

# Add presence tests
for f in os.listdir("scripts-pres"):
    tests.append("mod_pres.py scripts-pres/" + f)

# Add mod_sendto tests
for f in os.listdir("scripts-sendto"):
    tests.append("mod_sendto.py scripts-sendto/" + f)

# Add mod_media_playrec tests
for f in os.listdir("scripts-media-playrec"):
    tests.append("mod_media_playrec.py scripts-media-playrec/" + f)

# Add mod_pesq tests
for f in os.listdir("scripts-pesq"):
    tests.append("mod_pesq.py scripts-pesq/" + f)

# Add recvfrom tests
for f in os.listdir("scripts-recvfrom"):
    tests.append("mod_recvfrom.py scripts-recvfrom/" + f)

# Add sipp tests
for f in os.listdir("scripts-sipp"):
    if f.endswith(".xml"):
        tests.append("mod_sipp.py scripts-sipp/" + f)


resume_script=""
retry_num=0
shell_cmd=""
with_log=True

# Parse arguments
sys.argv.pop(0)
while len(sys.argv):
    if sys.argv[0]=='/h' or sys.argv[0]=='-h' or sys.argv[0]=='--help' or sys.argv[0]=='/help':
        sys.argv.pop(0)
        print("Usage:")
        print("  runall.py [OPTIONS] [run.py-OPTIONS]")
        print("OPTIONS:")
        print("  --list")
        print("    List the tests")
        print("  --list-xml")
        print("    List the tests as XML format suitable for ccdash")
        print("  --resume,-r RESUME")
        print("    RESUME is string/substring to specify where to resume tests.")
        print("    If this argument is omited, tests will start from the beginning.")
        print("  --disable,-d TEST_NAME")
        print("    Disable a specific test that contains the specified TEST_NAME.")
        print("  --retry,-t RETRY_NUM")
        print("    Retry a specific test RETRY_NUM times before marking it as failed.")
        print("    Default is 0 (no retry).")
        print("  --shell,-s SHELL")
        print("    Run the tests with the specified SHELL cmd. This can also be")
        print("    used to run the test with ccdash. Example:")
        print("    --shell '/bin/sh -c'")
        print("  --no-log")
        print("    Do not generate log files. By default log files will be generated")
        print("    and put in 'logs' dir.")
        print("")
        print("  run.py-OPTIONS are applicable here")
        sys.exit(0)
    elif sys.argv[0] == '-r' or sys.argv[0] == '--resume':
        if len(sys.argv) > 1:
            resume_script=sys.argv[1]
            sys.argv.pop(0)
            sys.argv.pop(0)
        else:
            sys.argv.pop(0)
            sys.stderr.write("Error: argument value required")
            sys.exit(1)
    elif sys.argv[0] == '-t' or sys.argv[0] == '--retry':
        if len(sys.argv) > 1:
            retry_num=int(sys.argv[1])
            sys.argv.pop(0)
            sys.argv.pop(0)
        else:
            sys.argv.pop(0)
            sys.stderr.write("Error: argument value required")
            sys.exit(1)
    elif sys.argv[0] == '--list':
        sys.argv.pop(0)
        for t in tests:
              print(t)
        sys.exit(0)
    elif sys.argv[0] == '--list-xml':
        sys.argv.pop(0)
        for t in tests:
            (mod,param) = t.split(None,2)
            tname = mod[4:mod.find(".py")] + "_" + \
            param[param.find("/")+1:param.rfind(".")]
            c = ""
            if len(sys.argv):
                c = " ".join(sys.argv) + " "
            tcmd = PYTHON + ' run.py ' + c + t
            print('\t\t<Test name="%s" cmd="%s" wdir="tests/pjsua" />' % (tname, tcmd))
        sys.exit(0)
    elif sys.argv[0] == '-s' or sys.argv[0] == '--shell':
        if len(sys.argv) > 1:
            shell_cmd = sys.argv[1]
            sys.argv.pop(0)
            sys.argv.pop(0)
        else:
            sys.argv.pop(0)
            sys.stderr.write("Error: argument value required")
            sys.exit(1)
    elif sys.argv[0] == '-d' or sys.argv[0] == '--disable':
        if len(sys.argv) > 1:
            excluded_tests.append(sys.argv[1])
            sys.argv.pop(0)
            sys.argv.pop(0)
        else:
            sys.argv.pop(0)
            sys.stderr.write("Error: argument value required")
            sys.exit(1)
    elif sys.argv[0] == '--no-log':
        sys.argv.pop(0)
        with_log=False
    else:
        # should be run.py options
        break


# Generate arguments for run.py
argv_st = " ".join(sys.argv) + " "

# Init vars
fails_cnt = 0
tests_cnt = 0

# Filter-out excluded tests
for pat in excluded_tests:
    tests = [t for t in tests if t.find(pat)==-1]

# Now run the tests
total_cnt = len(tests)
for t in tests:
    if resume_script!="" and t.find(resume_script)==-1:
        print("Skipping " + t +"..")
        total_cnt = total_cnt - 1
        continue
    resume_script=""
    cmdline = "python run.py " + argv_st + t
    if shell_cmd:
        cmdline = "%s '%s'" % (shell_cmd, cmdline)
    for i in range(1, retry_num+2):
        t0 = time.time()
        msg = "Running %3d/%d (#%d): %s..." % (tests_cnt+1, total_cnt, i, cmdline)
        sys.stdout.write(msg)
        sys.stdout.flush()
        if with_log:
            logname = re.search(".*\s+(.*)", t).group(1)
            logname = re.sub("[\\\/]", "_", logname)
            logname = re.sub("\.py$", ".log", logname)
            logname = re.sub("\.xml$", ".log", logname)
            logname = "logs/" + logname
        else:
            logname = os.devnull
        ret = os.system(cmdline + " > " + logname)
        t1 = time.time()
        if ret != 0:
            dur = int(t1 - t0)
            print(" failed!! [" + str(dur) + "s]")
            if (i < retry_num + 1):
                continue
            if with_log:
                lines = open(logname, "r", encoding='utf-8', errors='ignore').readlines()
                print(''.join(lines))
                print("Log file: '" + logname + "'.")
            fails_cnt += 1
        else:
            dur = int(t1 - t0)
            print(" ok [" + str(dur) + "s]")
            break
    tests_cnt += 1

if fails_cnt == 0:
    print("All " + str(tests_cnt) + " tests completed successfully")
else:
    print(str(tests_cnt) + " tests completed, " +  str(fails_cnt) + " test(s) failed")

sys.exit(fails_cnt)



#
from inc_cfg import *

# This test:
#   to make call with ICE but without STUN.

# Note:
#	- need --dis-codec to make INVITE packet less than typical MTU
uas_args = "--null-audio --id=\"<sip:test1@pjsip.org>\" --registrar=sip:sip.pjsip.org --username=test1 --password=test1  --realm=pjsip.org  --proxy=\"sip:sip.pjsip.org;lr\"  --rtp-port 0 --use-ice --use-compact-form --max-calls 1 --dis-codec=i --dis-codec=s --dis-codec=g --log-file callee.log"

uac_args = "--null-audio --id=\"<sip:test2@pjsip.org>\" --registrar=sip:sip.pjsip.org --username=test2 --password=test2 --realm=pjsip.org --proxy=\"sip:sip.pjsip.org;lr\" --rtp-port 0 --use-ice --use-compact-form --max-calls 1 --dis-codec=i --dis-codec=s --dis-codec=g --log-file caller.log"

test_param = TestParam(
		"ICE via public internet with no STUN",
		[
			InstanceParam(	"callee", uas_args, 
					uri="<sip:test1@pjsip.org>",
					have_reg=True, have_publish=False),
			InstanceParam(	"caller", uac_args,
					uri="<sip:test2@pjsip.org>",
					have_reg=True, have_publish=False),
		]
		)




import inc_sip as sip
import inc_sdp as sdp

# Authentication failure test with same nonce


pjsua = "--null-audio --id=sip:CLIENT --registrar sip:127.0.0.1:$PORT " + \
	"--realm=python --user=username --password=password"

req1 = sip.RecvfromTransaction("Initial request", 401,
				include=["REGISTER sip"], 
				exclude=["Authorization"],
				resp_hdr=["WWW-Authenticate: Digest realm=\"python\", nonce=\"1\""]
			  	)

req2 = sip.RecvfromTransaction("REGISTER retry", 401,
				include=["REGISTER sip", "Authorization", "nonce=\"1\""], 
				exclude=["Authorization:[\\s\\S]+Authorization:"],
				resp_hdr=["WWW-Authenticate: Digest realm=\"python\", nonce=\"1\""],
				expect="PJSIP_EFAILEDCREDENTIAL"
			  	)


recvfrom_cfg = sip.RecvfromCfg("Authentication failure with same nonce",
			       pjsua, [req1, req2])



#
import inc_const as const

PJSUA = ["--null-audio --max-calls=1 --no-tcp --auto-answer=183"]

PJSUA_EXPECTS = [[0, "Response msg 488/INVITE", ""]]

PJSUA_CLI_EXPECTS = [[0, "Response msg 488/INVITE", ""]]



import inc_sip as sip
import inc_sdp as sdp

# Torture message from RFC 4475
# 3.1.1.  Valid Messages
# 3.1.1.2.  Wide Range of Valid Characters
complete_msg = \
"""!interesting-Method0123456789_*+`.%indeed'~ sip:1_unusual.URI~(to-be!sure)&isn't+it$/crazy?,/;;*:&it+has=1,weird!*pas$wo~d_too.(doesn't-it)@example.com SIP/2.0
Via: SIP/2.0/UDP host1.example.com;rport;branch=z9hG4bK-.!%66*_+`'~
To: "BEL:\\\x07 NUL:\\\x00 DEL:\\\x7F" <sip:1_unusual.URI~(to-be!sure)&isn't+it$/crazy?,/;;*@example.com>
From: token1~` token2'+_ token3*%!.- <sip:mundane@example.com> ;fromParam''~+*_!.-%="\xD1\x80\xD0\xB0\xD0\xB1\xD0\xBE\xD1\x82\xD0\xB0\xD1\x8E\xD1\x89\xD0\xB8\xD0\xB9";tag=_token~1'+`*%!-.
Call-ID: intmeth.word%ZK-!.*_+'@word`~)(><:\\/"][?}{
CSeq: 139122385 !interesting-Method0123456789_*+`.%indeed'~
Max-Forwards: 255
extensionHeader-!.%*+_`'~: \xEF\xBB\xBF\xE5\xA4\xA7\xE5\x81\x9C\xE9\x9B\xBB
Content-Length: 0

"""


sendto_cfg = sip.SendtoCfg( "RFC 4475 3.1.1.2", 
			    "--null-audio --auto-answer 200", 
			    "", 405, complete_msg=complete_msg)




#
# SIP call sample.
#
# Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
#
import sys
import pjsua as pj

LOG_LEVEL=3
current_call = None

# Logging callback
def log_cb(level, str, len):
    print str,


# Callback to receive events from account
class MyAccountCallback(pj.AccountCallback):

    def __init__(self, account=None):
        pj.AccountCallback.__init__(self, account)

    # Notification on incoming call
    def on_incoming_call(self, call):
        global current_call 
        if current_call:
            call.answer(486, "Busy")
            return
            
        print "Incoming call from ", call.info().remote_uri
        print "Press 'a' to answer"

        current_call = call

        call_cb = MyCallCallback(current_call)
        current_call.set_callback(call_cb)

        current_call.answer(180)

        
# Callback to receive events from Call
class MyCallCallback(pj.CallCallback):

    def __init__(self, call=None):
        pj.CallCallback.__init__(self, call)

    # Notification when call state has changed
    def on_state(self):
        global current_call
        print "Call with", self.call.info().remote_uri,
        print "is", self.call.info().state_text,
        print "last code =", self.call.info().last_code, 
        print "(" + self.call.info().last_reason + ")"
        
        if self.call.info().state == pj.CallState.DISCONNECTED:
            current_call = None
            print 'Current call is', current_call

    # Notification when call's media state has changed.
    def on_media_state(self):
        if self.call.info().media_state == pj.MediaState.ACTIVE:
            # Connect the call to sound device
            call_slot = self.call.info().conf_slot
            pj.Lib.instance().conf_connect(call_slot, 0)
            pj.Lib.instance().conf_connect(0, call_slot)
            print "Media is now active"
        else:
            print "Media is inactive"

# Function to make call
def make_call(uri):
    try:
        print "Making call to", uri
        return acc.make_call(uri, cb=MyCallCallback())
    except pj.Error, e:
        print "Exception: " + str(e)
        return None
        

# Create library instance
lib = pj.Lib()

try:
    # Init library with default config and some customized
    # logging config.
    lib.init(log_cfg = pj.LogConfig(level=LOG_LEVEL, callback=log_cb))

    # Create UDP transport which listens to any available port
    transport = lib.create_transport(pj.TransportType.UDP, 
                                     pj.TransportConfig(0))
    print "\nListening on", transport.info().host, 
    print "port", transport.info().port, "\n"
    
    # Start the library
    lib.start()

    # Create local account
    acc = lib.create_account_for_transport(transport, cb=MyAccountCallback())

    # If argument is specified then make call to the URI
    if len(sys.argv) > 1:
        lck = lib.auto_lock()
        current_call = make_call(sys.argv[1])
        print 'Current call is', current_call
        del lck

    my_sip_uri = "sip:" + transport.info().host + \
                 ":" + str(transport.info().port)

    # Menu loop
    while True:
        print "My SIP URI is", my_sip_uri
        print "Menu:  m=make call, h=hangup call, a=answer call, q=quit"

        input = sys.stdin.readline().rstrip("\r\n")
        if input == "m":
            if current_call:
                print "Already have another call"
                continue
            print "Enter destination URI to call: ", 
            input = sys.stdin.readline().rstrip("\r\n")
            if input == "":
                continue
            lck = lib.auto_lock()
            current_call = make_call(input)
            del lck

        elif input == "h":
            if not current_call:
                print "There is no call"
                continue
            current_call.hangup()

        elif input == "a":
            if not current_call:
                print "There is no call"
                continue
            current_call.answer(200)

        elif input == "q":
            break

    # Shutdown the library
    transport = None
    acc.delete()
    acc = None
    lib.destroy()
    lib = None

except pj.Error, e:
    print "Exception: " + str(e)
    lib.destroy()
    lib = None




#
import inc_const as const

PJSUA = ["--null-audio",    # UA0
	 "--null-audio",    # UA1
	 "--null-audio"	    # UA2
	]

PJSUA_EXPECTS = [
		 # A calls B
		 [0, "", "m"],
		 [0, "", "$PJSUA_URI[1]"],
		 [0, const.STATE_CALLING, ""],
		 [1, const.EVENT_INCOMING_CALL, "a"],
		 [1, "", "200"],
		 [0, const.STATE_CONFIRMED, ""],
		 [1, const.STATE_CONFIRMED, ""],

		 # B transfer A to C
		 [1, "", "x"],
		 [1, "", "$PJSUA_URI[2]"],
		 [0, const.STATE_CALLING, ""],
		 [2, const.EVENT_INCOMING_CALL, "a"],
		 [2, "", "200"],
		 [0, const.MEDIA_ACTIVE, ""],
		 [2, const.MEDIA_ACTIVE, ""],
		 [1, "call transferred successfully", ""],
		 [1, const.STATE_DISCONNECTED, ""]
		]

PJSUA_CLI_EXPECTS = [
		 # A calls B
		 [0, "", "call new $PJSUA_URI[1]"],
		 [0, const.STATE_CALLING, ""],
		 [1, const.EVENT_INCOMING_CALL, "call answer 200"],
		 [0, const.STATE_CONFIRMED, ""],
		 [1, const.STATE_CONFIRMED, ""],

		 # B transfer A to C
		 [1, "", "call transfer $PJSUA_URI[2]"],
		 [0, const.STATE_CALLING, ""],
		 [2, const.EVENT_INCOMING_CALL, "call answer 200"],
		 [0, const.MEDIA_ACTIVE, ""],
		 [2, const.MEDIA_ACTIVE, ""],
		 [1, "call transferred successfully", ""],
		 [1, const.STATE_DISCONNECTED, ""]
		]



#
from inc_cfg import *

# Basic registration
test_param = TestParam(
		"Basic registration",
		[
			InstanceParam(	"client", 
					"--null-audio"+
						" --id=\"<sip:test1@pjsip.org>\""+
						" --registrar=sip:sip.pjsip.org" +
						" --username=test1" +
						" --password=test1" +
						" --realm=*",
					uri="sip:test1@pjsip.org",
					have_reg=True),
		]
		)




#
from inc_cfg import *

# Call with default pjsua settings
test_param = TestParam(
		"PESQ defaults pjsua settings (RX side uses snd dev)",
		[
			InstanceParam("UA1", "--max-calls=1 --play-file wavs/input.16.wav --null-audio"),
			InstanceParam("UA2", "--max-calls=1 --rec-file  wavs/tmp.16.wav --clock-rate 16000 --auto-answer 200")
		]
		)


if (HAS_SND_DEV == 0):
	test_param.skip = True

pesq_threshold = None



import inc_sip as sip
import inc_sdp as sdp

sdp = \
"""
v=
o=
s=
c=
t=
a=
"""

pjsua_args = "--null-audio --auto-answer 200"
extra_headers = ""
include = [ "Warning: " ]	# better have Warning header
exclude = []

sendto_cfg = sip.SendtoCfg("Bad SDP syntax", pjsua_args, sdp, 400,
			   extra_headers=extra_headers,
			   resp_inc=include, resp_exc=exclude) 




#
from inc_cfg import *

ADD_PARAM = ""

if (HAS_SND_DEV == 0):
	ADD_PARAM += "--null-audio"

# Call with default pjsua settings
test_param = TestParam(
		"PESQ defaults pjsua settings",
		[
			InstanceParam("UA1", ADD_PARAM + " --max-calls=1 --play-file wavs/input.16.wav --no-vad"),
			InstanceParam("UA2", "--null-audio --max-calls=1 --rec-file  wavs/tmp.16.wav --clock-rate 16000 --auto-answer 200")
		]
		)

pesq_threshold = 3.8



import inc_sip as sip
import inc_sdp as sdp

body = \
"""
This is the preamble.  It is to be ignored, though it
is a handy place for composition agents to include an
explanatory note to non-MIME conformant readers.

--123:45
Content-Type: text/plain

The first part is definitely not SDP

--123:45

This is implicitly typed plain US-ASCII text.
It does NOT end with a linebreak.
--123:45
Content-Type: application/sdp

v=0
o=- 0 0 IN IP4 127.0.0.1
s=pjmedia
c=IN IP4 127.0.0.1
t=0 0
m=audio 4000 RTP/AVP 0 101
a=rtpmap:0 PCMU/8000
a=sendrecv
a=rtpmap:101 telephone-event/8000
a=fmtp:101 0-15

--123:45--
This is the epilogue.  It is also to be ignored.
"""

args = "--null-audio --auto-answer 200 --max-calls 1"
extra_headers = "Content-Type: multipart/mixed; boundary=\"123:45\""
include = ["v=0", "m=audio"]
exclude = []

sendto_cfg = sip.SendtoCfg( "Valid but cluttered multipart/mixed body containing SDP", 
			    pjsua_args=args, sdp="", resp_code=200, 
			    extra_headers=extra_headers, body=body,
			    resp_inc=include, resp_exc=exclude)




#
# pjsua Python GUI Demo
#
# Copyright (C)2013 Teluu Inc. (http://www.teluu.com)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
#
import sys
if sys.version_info[0] >= 3: # Python 3
	import tkinter as tk
	from tkinter import ttk
	from tkinter import messagebox as msgbox
else:
	import Tkinter as tk
	import tkMessageBox as msgbox
	import ttk

import pjsua2 as pj
#import application

# Transport setting
class SipTransportConfig:
	def __init__(self, type, enabled):
		#pj.PersistentObject.__init__(self)
		self.type = type
		self.enabled = enabled
		self.config = pj.TransportConfig()
	def readObject(self, node):
		child_node = node.readContainer("SipTransport")
		self.type = child_node.readInt("type")
		self.enabled = child_node.readBool("enabled")
		self.config.readObject(child_node)
	def writeObject(self, node):
		child_node = node.writeNewContainer("SipTransport")
		child_node.writeInt("type", self.type)
		child_node.writeBool("enabled", self.enabled)
		self.config.writeObject(child_node)

# Account setting with buddy list
class AccConfig:
	def __init__(self):
		self.enabled = True
		self.config = pj.AccountConfig()
		self.buddyConfigs = []
	def readObject(self, node):
		acc_node = node.readContainer("Account")
		self.enabled = acc_node.readBool("enabled")
		self.config.readObject(acc_node)
		buddy_node = acc_node.readArray("buddies")
		while buddy_node.hasUnread():
			buddy_cfg = pj.BuddyConfig()
			buddy_cfg.readObject(buddy_node)
			self.buddyConfigs.append(buddy_cfg)
	def writeObject(self, node):
		acc_node = node.writeNewContainer("Account")
		acc_node.writeBool("enabled", self.enabled)
		self.config.writeObject(acc_node)
		buddy_node = acc_node.writeNewArray("buddies")
		for buddy in self.buddyConfigs:
			buddy_node.writeObject(buddy)

	
# Master settings
class AppConfig:
	def __init__(self):
		self.epConfig = pj.EpConfig()	# pj.EpConfig()
		self.udp = SipTransportConfig(pj.PJSIP_TRANSPORT_UDP, True)
		self.tcp = SipTransportConfig(pj.PJSIP_TRANSPORT_TCP, True)
		self.tls = SipTransportConfig(pj.PJSIP_TRANSPORT_TLS, False)
		self.accounts = []		# Array of AccConfig
		
	def loadFile(self, file):
		json = pj.JsonDocument()
		json.loadFile(file)
		root = json.getRootContainer()
		self.epConfig = pj.EpConfig()
		self.epConfig.readObject(root)
		
		tp_node = root.readArray("transports")
		self.udp.readObject(tp_node)
		self.tcp.readObject(tp_node)
		if tp_node.hasUnread():
			self.tls.readObject(tp_node)
			
		acc_node = root.readArray("accounts")
		while acc_node.hasUnread():
			acfg = AccConfig()
			acfg.readObject(acc_node)
			self.accounts.append(acfg)
			
	def saveFile(self,file):
		json = pj.JsonDocument()
		
		# Write endpoint config
		json.writeObject(self.epConfig)
		
		# Write transport config
		tp_node = json.writeNewArray("transports")
		self.udp.writeObject(tp_node)
		self.tcp.writeObject(tp_node)
		self.tls.writeObject(tp_node)
		
		# Write account configs
		node = json.writeNewArray("accounts")
		for acc in self.accounts:
			acc.writeObject(node)
				
		json.saveFile(file)
		

# Settings dialog
class Dialog(tk.Toplevel):
	"""
	This implements account settings dialog to manipulate account settings.
	"""
	def __init__(self, parent, cfg):
		tk.Toplevel.__init__(self, parent)
		self.transient(parent)
		self.parent = parent
		self.title('Settings')
		
		self.frm = ttk.Frame(self)
		self.frm.pack(expand='yes', fill='both')
		
		self.isOk = False
		self.cfg = cfg
		
		self.createWidgets()
	
	def doModal(self):
		if self.parent:
			self.parent.wait_window(self)
		else:
			self.wait_window(self)
		return self.isOk
		
	def createWidgets(self):
		# The notebook
		self.frm.rowconfigure(0, weight=1)
		self.frm.rowconfigure(1, weight=0)
		self.frm.columnconfigure(0, weight=1)
		self.frm.columnconfigure(1, weight=1)
		self.wTab = ttk.Notebook(self.frm)
		self.wTab.grid(column=0, row=0, columnspan=2, padx=10, pady=10, ipadx=20, ipady=20, sticky=tk.N+tk.S+tk.W+tk.E)
		
		# Main buttons
		btnOk = ttk.Button(self.frm, text='Ok', command=self.onOk)
		btnOk.grid(column=0, row=1, sticky=tk.E, padx=20, pady=10)
		btnCancel = ttk.Button(self.frm, text='Cancel', command=self.onCancel)
		btnCancel.grid(column=1, row=1, sticky=tk.W, padx=20, pady=10)
		
		# Tabs
		self.createBasicTab()
		self.createNetworkTab()
		self.createMediaTab()

	def createBasicTab(self):
		# Prepare the variables to set/receive values from GUI
		self.cfgLogFile = tk.StringVar(value=self.cfg.epConfig.logConfig.filename)
		self.cfgLogAppend = tk.BooleanVar(value=True if (self.cfg.epConfig.logConfig.fileFlags & pj.PJ_O_APPEND) else False)
		
		# Build the tab page
		frm = ttk.Frame(self.frm)
		frm.columnconfigure(0, weight=1)
		frm.columnconfigure(1, weight=2)
		row = 0
		ttk.Label(frm, text='User Agent:').grid(row=row, column=0, sticky=tk.E, pady=2, padx=8)
		ttk.Label(frm, text=self.cfg.epConfig.uaConfig.userAgent).grid(row=row, column=1, sticky=tk.W, pady=2, padx=6)
		row += 1
		ttk.Label(frm, text='Max calls:').grid(row=row, column=0, sticky=tk.E, pady=2, padx=8)
		ttk.Label(frm, text=str(self.cfg.epConfig.uaConfig.maxCalls)).grid(row=row, column=1, sticky=tk.W, pady=2, padx=6)
		row += 1
		ttk.Label(frm, text='Log file:').grid(row=row, column=0, sticky=tk.E, pady=2, padx=8)
		ttk.Entry(frm, textvariable=self.cfgLogFile, width=32).grid(row=row, column=1, sticky=tk.W, padx=6)
		row += 1
		ttk.Checkbutton(frm, text='Append log file', variable=self.cfgLogAppend).grid(row=row, column=1, sticky=tk.W, padx=6, pady=2)

		self.wTab.add(frm, text='Basic')
		
	def createNetworkTab(self):
		self.cfgNameserver = tk.StringVar()
		if len(self.cfg.epConfig.uaConfig.nameserver):
			self.cfgNameserver.set(self.cfg.epConfig.uaConfig.nameserver[0])
		self.cfgStunServer = tk.StringVar()
		if len(self.cfg.epConfig.uaConfig.stunServer):
			self.cfgStunServer.set(self.cfg.epConfig.uaConfig.stunServer[0])
		self.cfgStunIgnoreError = tk.BooleanVar(value=self.cfg.epConfig.uaConfig.stunIgnoreFailure)
		
		self.cfgUdpEnabled = tk.BooleanVar(value=self.cfg.udp.enabled)
		self.cfgUdpPort = tk.IntVar(value=self.cfg.udp.config.port)
		self.cfgTcpEnabled = tk.BooleanVar(value=self.cfg.tcp.enabled)
		self.cfgTcpPort = tk.IntVar(value=self.cfg.tcp.config.port)
		self.cfgTlsEnabled = tk.BooleanVar(value=self.cfg.tls.enabled)
		self.cfgTlsPort = tk.IntVar(value=self.cfg.tls.config.port)
		
		self.cfgTlsCaFile = tk.StringVar(value=self.cfg.tls.config.tlsConfig.CaListFile)
		self.cfgTlsCertFile = tk.StringVar(value=self.cfg.tls.config.tlsConfig.certFile)
		self.cfgTlsVerifyClient = tk.BooleanVar(value=self.cfg.tls.config.tlsConfig.verifyClient)
		self.cfgTlsVerifyServer = tk.BooleanVar(value=self.cfg.tls.config.tlsConfig.verifyServer)
		
		# Build the tab page
		frm = ttk.Frame(self.frm)
		frm.columnconfigure(0, weight=1)
		frm.columnconfigure(1, weight=2)
		row = 0
		#ttk.Label(frm, text='UDP transport:').grid(row=row, column=0, sticky=tk.E, pady=2, padx=8)
		ttk.Checkbutton(frm, text='Enable UDP transport', variable=self.cfgUdpEnabled).grid(row=row, column=0, sticky=tk.W, padx=6, pady=2)
		row += 1
		ttk.Label(frm, text='UDP port:').grid(row=row, column=0, sticky=tk.E, pady=2, padx=8)
		tk.Spinbox(frm, from_=0, to=65535, textvariable=self.cfgUdpPort, width=5).grid(row=row, column=1, sticky=tk.W, padx=6)
		ttk.Label(frm, text='(0 for any)').grid(row=row, column=1, sticky=tk.E, pady=6, padx=6)
		row += 1
		#ttk.Label(frm, text='TCP transport:').grid(row=row, column=0, sticky=tk.E, pady=2, padx=8)
		ttk.Checkbutton(frm, text='Enable TCP transport', variable=self.cfgTcpEnabled).grid(row=row, column=0, sticky=tk.W, padx=6, pady=2)
		row += 1
		ttk.Label(frm, text='TCP port:').grid(row=row, column=0, sticky=tk.E, pady=2, padx=8)
		tk.Spinbox(frm, from_=0, to=65535, textvariable=self.cfgTcpPort, width=5).grid(row=row, column=1, sticky=tk.W, padx=6)
		ttk.Label(frm, text='(0 for any)').grid(row=row, column=1, sticky=tk.E, pady=6, padx=6)
		row += 1
		#ttk.Label(frm, text='TLS transport:').grid(row=row, column=0, sticky=tk.E, pady=2, padx=8)
		ttk.Checkbutton(frm, text='Enable TLS transport', variable=self.cfgTlsEnabled).grid(row=row, column=0, sticky=tk.W, padx=6, pady=2)
		row += 1
		ttk.Label(frm, text='TLS port:').grid(row=row, column=0, sticky=tk.E, pady=2, padx=8)
		tk.Spinbox(frm, from_=0, to=65535, textvariable=self.cfgTlsPort, width=5).grid(row=row, column=1, sticky=tk.W, padx=6)
		ttk.Label(frm, text='(0 for any)').grid(row=row, column=1, sticky=tk.E, pady=6, padx=6)
		row += 1
		ttk.Label(frm, text='TLS CA file:').grid(row=row, column=0, sticky=tk.E, pady=2, padx=8)
		ttk.Entry(frm, textvariable=self.cfgTlsCaFile, width=32).grid(row=row, column=1, sticky=tk.W, padx=6)
		row += 1
		ttk.Label(frm, text='TLS cert file:').grid(row=row, column=0, sticky=tk.E, pady=2, padx=8)
		ttk.Entry(frm, textvariable=self.cfgTlsCertFile, width=32).grid(row=row, column=1, sticky=tk.W, padx=6)
		row += 1
		ttk.Checkbutton(frm, text='TLS verify server', variable=self.cfgTlsVerifyServer).grid(row=row, column=1, sticky=tk.W, padx=6, pady=2)
		row += 1
		ttk.Checkbutton(frm, text='TLS verify client', variable=self.cfgTlsVerifyClient).grid(row=row, column=1, sticky=tk.W, padx=6, pady=2)
		row += 1
		ttk.Label(frm, text='DNS and STUN:').grid(row=row, column=0, sticky=tk.W, pady=2, padx=8)
		row += 1
		ttk.Label(frm, text='Nameserver:').grid(row=row, column=0, sticky=tk.E, pady=2, padx=8)
		ttk.Entry(frm, textvariable=self.cfgNameserver, width=32).grid(row=row, column=1, sticky=tk.W, padx=6)
		row += 1
		ttk.Label(frm, text='STUN Server:').grid(row=row, column=0, sticky=tk.E, pady=2, padx=8)
		ttk.Entry(frm, textvariable=self.cfgStunServer, width=32).grid(row=row, column=1, sticky=tk.W, padx=6)
		row += 1
		ttk.Checkbutton(frm, text='Ignore STUN failure at startup', variable=self.cfgStunIgnoreError).grid(row=row, column=1, sticky=tk.W, padx=6, pady=2)
		
		self.wTab.add(frm, text='Network')

	def createMediaTab(self):
		self.cfgClockrate = tk.IntVar(value=self.cfg.epConfig.medConfig.clockRate)
		self.cfgSndClockrate = tk.IntVar(value=self.cfg.epConfig.medConfig.sndClockRate)
		self.cfgAudioPtime = tk.IntVar(value=self.cfg.epConfig.medConfig.audioFramePtime)
		self.cfgMediaQuality = tk.IntVar(value=self.cfg.epConfig.medConfig.quality)
		self.cfgCodecPtime = tk.IntVar(value=self.cfg.epConfig.medConfig.ptime)
		self.cfgVad = tk.BooleanVar(value=not self.cfg.epConfig.medConfig.noVad)
		self.cfgEcTailLen = tk.IntVar(value=self.cfg.epConfig.medConfig.ecTailLen)
		
		# Build the tab page
		frm = ttk.Frame(self.frm)
		frm.columnconfigure(0, weight=1)
		frm.columnconfigure(1, weight=2)
		row = 0
		ttk.Label(frm, text='Max media ports:').grid(row=row, column=0, sticky=tk.E, pady=2, padx=8)
		ttk.Label(frm, text=str(self.cfg.epConfig.medConfig.maxMediaPorts)).grid(row=row, column=1, sticky=tk.W, pady=2, padx=6)
		row += 1
		ttk.Label(frm, text='Core clock rate:').grid(row=row, column=0, sticky=tk.E, pady=2, padx=8)
		tk.Spinbox(frm, from_=8000, to=48000, increment=8000, textvariable=self.cfgClockrate, width=5).grid(row=row, column=1, sticky=tk.W, padx=6)
		row += 1
		ttk.Label(frm, text='Snd device clock rate:').grid(row=row, column=0, sticky=tk.E, pady=2, padx=8)
		tk.Spinbox(frm, from_=0, to=48000, increment=8000, textvariable=self.cfgSndClockrate, width=5).grid(row=row, column=1, sticky=tk.W, padx=6)
		ttk.Label(frm, text='(0: follow core)').grid(row=row, column=1, sticky=tk.E, pady=6, padx=6)
		row += 1
		ttk.Label(frm, text='Core ptime:').grid(row=row, column=0, sticky=tk.E, pady=2, padx=8)
		tk.Spinbox(frm, from_=10, to=400, increment=10, textvariable=self.cfgAudioPtime, width=3).grid(row=row, column=1, sticky=tk.W, padx=6)
		row += 1
		ttk.Label(frm, text='RTP ptime:').grid(row=row, column=0, sticky=tk.E, pady=2, padx=8)
		tk.Spinbox(frm, from_=20, to=400, increment=10, textvariable=self.cfgCodecPtime, width=3).grid(row=row, column=1, sticky=tk.W, padx=6)
		row += 1
		ttk.Label(frm, text='Media quality (1-10):').grid(row=row, column=0, sticky=tk.E, pady=2, padx=8)
		tk.Spinbox(frm, from_=1, to=10, textvariable=self.cfgMediaQuality, width=5).grid(row=row, column=1, sticky=tk.W, padx=6)
		row += 1
		ttk.Label(frm, text='VAD:').grid(row=row, column=0, sticky=tk.E, pady=2, padx=8)
		ttk.Checkbutton(frm, text='Enable', variable=self.cfgVad).grid(row=row, column=1, sticky=tk.W, padx=6, pady=2)
		row += 1
		ttk.Label(frm, text='Echo canceller tail length:').grid(row=row, column=0, sticky=tk.E, pady=2, padx=8)
		tk.Spinbox(frm, from_=0, to=400, increment=10, textvariable=self.cfgEcTailLen, width=3).grid(row=row, column=1, sticky=tk.W, padx=6)
		ttk.Label(frm, text='(ms, 0 to disable)').grid(row=row, column=1, sticky=tk.E, pady=6, padx=6)
		
		self.wTab.add(frm, text='Media')

	def onOk(self):
		# Check basic settings
		errors = "";
		if errors:
			msgbox.showerror("Error detected:", errors)
			return
		
		# Basic settings
		self.cfg.epConfig.logConfig.filename = self.cfgLogFile.get()
		flags = pj.PJ_O_APPEND if self.cfgLogAppend.get() else 0
		self.cfg.epConfig.logConfig.fileFlags = self.cfg.epConfig.logConfig.fileFlags | flags 
		
		# Network settings
		self.cfg.epConfig.uaConfig.nameserver.clear()
		if len(self.cfgNameserver.get()): 
			self.cfg.epConfig.uaConfig.nameserver.append(self.cfgNameserver.get())
		self.cfg.epConfig.uaConfig.stunServer.clear()
		if len(self.cfgStunServer.get()):
			self.cfg.epConfig.uaConfig.stunServer.append(self.cfgStunServer.get())
			
		self.cfg.epConfig.uaConfig.stunIgnoreFailure = self.cfgStunIgnoreError.get()
		
		self.cfg.udp.enabled 	= self.cfgUdpEnabled.get()
		self.cfg.udp.config.port = self.cfgUdpPort.get()
		self.cfg.tcp.enabled 	= self.cfgTcpEnabled.get()
		self.cfg.tcp.config.port = self.cfgTcpPort.get()
		self.cfg.tls.enabled 	= self.cfgTlsEnabled.get()
		self.cfg.tls.config.port = self.cfgTlsPort.get()
		
		self.cfg.tls.config.tlsConfig.CaListFile = self.cfgTlsCaFile.get()
		self.cfg.tls.config.tlsConfig.certFile   = self.cfgTlsCertFile.get()
		self.cfg.tls.config.tlsConfig.verifyClient = self.cfgTlsVerifyClient.get()
		self.cfg.tls.config.tlsConfig.verifyServer = self.cfgTlsVerifyServer.get()

		# Media
		self.cfg.epConfig.medConfig.clockRate	= self.cfgClockrate.get()
		self.cfg.epConfig.medConfig.sndClockRate = self.cfgSndClockrate.get()
		self.cfg.epConfig.medConfig.audioFramePtime = self.cfgAudioPtime.get()
		self.cfg.epConfig.medConfig.quality	= self.cfgMediaQuality.get()
		self.cfg.epConfig.medConfig.ptime	= self.cfgCodecPtime.get()
		self.cfg.epConfig.medConfig.noVad	= not self.cfgVad.get()
		self.cfg.epConfig.medConfig.ecTailLen	= self.cfgEcTailLen.get()
		
		self.isOk = True
		self.destroy()
		
	def onCancel(self):
		self.destroy()


if __name__ == '__main__':
	#application.main()
	acfg = AppConfig()
	acfg.loadFile('pygui.js')

	dlg = Dialog(None, acfg)
	if dlg.doModal():
		acfg.saveFile('pygui.js')
		


import inc_sip as sip
import inc_sdp as sdp

# Ticket https://github.com/pjsip/pjproject/issues/718
# RTC doesn't put rport in Via, and it is reported to have caused segfault.
#
complete_msg = \
"""MESSAGE sip:localhost SIP/2.0
Via: SIP/2.0/UDP localhost:$LOCAL_PORT;branch=z9hG4bK$BRANCH
From: <sip:tester@localhost>;tag=as2858a32c
To: <sip:pjsua@localhost>
Call-ID: 123@localhost
CSeq: 1 MESSAGE
Max-Forwards: 70
Content-Length: 11
Content-Type: text/plain

Hello world
"""


sendto_cfg = sip.SendtoCfg( "RTC no rport", "--null-audio --auto-answer 200", 
			    "", 200, complete_msg=complete_msg)




#
from inc_cfg import *

# Call with PCMU codec
test_param = TestParam(
		"PESQ codec PCMU (RX side uses snd dev)",
		[
			InstanceParam("UA1", "--max-calls=1 --add-codec pcmu --clock-rate 8000 --play-file wavs/input.8.wav --null-audio"),
			InstanceParam("UA2", "--max-calls=1 --add-codec pcmu --clock-rate 8000 --rec-file  wavs/tmp.8.wav   --auto-answer 200")
		]
		)

if (HAS_SND_DEV == 0):
	test_param.skip = True

pesq_threshold = 3.5




# SDP template
sdp_templ = \
"""v=0\r
o=- 1 1 $NET_TYPE $ADDR_TYPE $LOCAL_IP\r
s=pjmedia\r
t=0 0\r
$SDP_LINES"""

sdp_media_templ = \
"""m=$MEDIA_TYPE $PORT $TRANSPORT 0\r
c=$NET_TYPE $ADDR_TYPE $LOCAL_IP\r
$SDP_LINES"""

# Create SDP session
def session(local_ip="127.0.0.1", extra_lines="", net_type="IN", addr_type="IP4"):
	sdp = sdp_templ
	sdp = sdp.replace("$NET_TYPE", net_type)
	sdp = sdp.replace("$ADDR_TYPE", addr_type)
	sdp = sdp.replace("$LOCAL_IP", local_ip)
	sdp = sdp.replace("$SDP_LINES", extra_lines)
	return sdp

# Create basic SDP media
def media(media_type="audio", local_port=4000, local_ip="127.0.0.1", extra_lines="", 
			  net_type = "IN", addr_type="IP4", transport="RTP/AVP"):
	sdp = sdp_media_templ
	sdp = sdp.replace("$MEDIA_TYPE", media_type)
	sdp = sdp.replace("$LOCAL_IP", local_ip)
	sdp = sdp.replace("$PORT", str(local_port))
	sdp = sdp.replace("$NET_TYPE", net_type)
	sdp = sdp.replace("$ADDR_TYPE", addr_type)
	sdp = sdp.replace("$TRANSPORT", transport)
	sdp = sdp.replace("$SDP_LINES", extra_lines)
	return sdp






# Quality test of media calls.
# - UA1 calls UA2
# - UA1 plays a file until finished to be streamed to UA2
# - UA2 records from stream
# - Apply PESQ to played file (reference) and recorded file (degraded)
#
# File should be:
# - naming: xxxxxx.CLOCK_RATE.wav, e.g: test1.8.wav
# - clock-rate of those files can only be 8khz or 16khz

import time
import os
import sys
import re
import subprocess
import wave
import shutil
import inc_const as const
import inc_util as util

from inc_cfg import *

# Load configuration
cfg_file = util.load_module_from_file("cfg_file", ARGS[1])

# PESQ configs
PESQ = "tools/pesq"				# PESQ executable path
PESQ_DEFAULT_THRESHOLD = 3.4	# Default minimum acceptable PESQ MOS value

# PESQ params
pesq_sample_rate_opt = ""		# Sample rate option for PESQ
input_filename    = ""			# Input/Reference filename
output_filename = ""			# Output/Degraded filename


# Test body function
def test_func(t):
    global pesq_sample_rate_opt
    global input_filename
    global output_filename

    ua1 = t.process[0]
    ua2 = t.process[1]

    # Get input file name
    input_filename = re.compile(const.MEDIA_PLAY_FILE).search(ua1.inst_param.arg).group(1)

    # Get output file name
    output_filename = re.compile(const.MEDIA_REC_FILE).search(ua2.inst_param.arg).group(1)

    # Get WAV input length, in seconds
    fin = wave.open(input_filename, "r")
    if fin == None:
        raise TestError("Failed opening input WAV file")
    inwavlen = fin.getnframes() * 1.0 / fin.getframerate()
    inwavlen += 0.2
    fin.close()
    print("WAV input len = " + str(inwavlen) + "s")

    # Get clock rate of the output
    mo_clock_rate = re.compile("\.(\d+)\.wav").search(output_filename)
    if (mo_clock_rate==None):
        raise TestError("Cannot compare input & output, incorrect output filename format")
    clock_rate = mo_clock_rate.group(1)
    
    # Get channel count of the output
    channel_count = 1
    if re.search("--stereo", ua2.inst_param.arg) != None:
        channel_count = 2
    
    # Get matched input file from output file
    # (PESQ evaluates only files whose same clock rate & channel count)
    if channel_count == 2:
        if re.search("\.\d+\.\d+\.wav", input_filename) != None:
            input_filename = re.sub("\.\d+\.\d+\.wav", "." + str(channel_count) + "."+clock_rate+".wav", input_filename)
        else:
            input_filename = re.sub("\.\d+\.wav", "." + str(channel_count) + "."+clock_rate+".wav", input_filename)

    if (clock_rate != "8") & (clock_rate != "16"):
        raise TestError("PESQ only works on clock rate 8kHz or 16kHz, clock rate used = "+clock_rate+ "kHz")

    # Get conference clock rate of UA2 for PESQ sample rate option
    pesq_sample_rate_opt = "+" + clock_rate + "000"

    # UA1 making call
    if ua1.use_telnet:
        ua1.send("call new " + t.inst_params[1].uri)
    else:
        ua1.send("m")
        ua1.send(t.inst_params[1].uri)
    ua1.expect(const.STATE_CALLING)

    # UA2 wait until call established
    ua2.expect(const.STATE_CONFIRMED)

    ua1.sync_stdout()
    ua2.sync_stdout()
    time.sleep(2)

    # Disconnect mic -> rec file, to avoid echo recorded when using sound device
    # Disconnect stream -> spk, make it silent
    # Connect stream -> rec file, start recording
    ua2.send("cd 0 1")
    ua2.send("cd 4 0")
    ua2.send("cc 4 1")

    # Disconnect mic -> stream, make stream purely sending from file
    # Disconnect stream -> spk, make it silent
    # Connect file -> stream, start sending
    ua1.send("cd 0 4")
    ua1.send("cd 4 0")
    ua1.send("cc 1 4")

    time.sleep(inwavlen)

    # Disconnect files from bridge
    ua2.send("cd 4 1")
    ua2.expect(const.MEDIA_DISCONN_PORT_SUCCESS)
    ua1.send("cd 1 4")
    ua1.expect(const.MEDIA_DISCONN_PORT_SUCCESS)


# Post body function
def post_func(t):
    global pesq_sample_rate_opt
    global input_filename
    global output_filename

    endpt = t.process[0]

    # Execute PESQ
    fullcmd = os.path.normpath(PESQ) + " " + pesq_sample_rate_opt + " " + input_filename + " " + output_filename
    endpt.trace("Popen " + fullcmd)
    pesq_proc = subprocess.Popen(fullcmd, shell=True, stdout=subprocess.PIPE, universal_newlines=True)
    pesq_out  = pesq_proc.communicate()

    # Parse ouput
    mo_pesq_out = re.compile("Prediction[^=]+=\s+([\-\d\.]+)\s*").search(pesq_out[0])
    if (mo_pesq_out == None):
        raise TestError("Failed to fetch PESQ result")

    # Get threshold
    if (cfg_file.pesq_threshold != None) | (cfg_file.pesq_threshold > -0.5 ):
        threshold = cfg_file.pesq_threshold
    else:
        threshold = PESQ_DEFAULT_THRESHOLD

    # Evaluate the PESQ MOS value
    pesq_res = mo_pesq_out.group(1)
    if (float(pesq_res) >= threshold):
        endpt.trace("Success, PESQ result = " + pesq_res + " (target=" + str(threshold) + ").")
    else:
        endpt.trace("Failed, PESQ result = " + pesq_res + " (target=" + str(threshold) + ").")
        # Save the wav file
        wavoutname = ARGS[1]
        wavoutname = re.sub("[\\\/]", "_", wavoutname)
        wavoutname = re.sub("\.py$", ".wav", wavoutname)
        wavoutname = "logs/" + wavoutname
        try:
            shutil.copyfile(output_filename, wavoutname)
            print("Output WAV is copied to " + wavoutname)
        except:
            print("Couldn't copy output WAV, please check if 'logs' directory exists.")

        raise TestError("WAV seems to be degraded badly, PESQ = "+ pesq_res + " (target=" + str(threshold) + ").")


# Here where it all comes together
test = cfg_file.test_param
test.test_func = test_func
test.post_func = post_func




#
from inc_cfg import *

# Call with L16/16000/2 codec
test_param = TestParam(
		"PESQ defaults pjsua settings",
		[
			InstanceParam("UA1", "--stereo --max-calls=1 --clock-rate 16000 --add-codec L16/16000/2 --play-file wavs/input.2.16.wav --null-audio"),
			InstanceParam("UA2", "--stereo --max-calls=1 --clock-rate 16000 --add-codec L16/16000/2 --rec-file  wavs/tmp.2.16.wav   --auto-answer 200")
		]
		)

if (HAS_SND_DEV == 0):
	test_param.skip = True
	
pesq_threshold = None



#
import inc_const as const

PJSUA = ["--null-audio --max-calls=1"]

PJSUA_EXPECTS = [[0, const.EVENT_INCOMING_CALL, "a"],
		 [0, "", "200"],
		 [0, const.MEDIA_ACTIVE, ""],
		 [0, const.STATE_CONFIRMED, "h"]
		 ]

PJSUA_CLI_EXPECTS = [[0, const.EVENT_INCOMING_CALL, "call answer 200"],
		 [0, const.MEDIA_ACTIVE, ""],
		 [0, const.STATE_CONFIRMED, "call hangup"]
		 ]



import inc_sip as sip
import inc_sdp as sdp

# Several PUBLISH failure scenarios that should be handled automatically


pjsua = "--null-audio --id=sip:127.0.0.1:$PORT --registrar sip:127.0.0.1:$PORT " + \
	"--realm=python --user=username --password=password " + \
	"--auto-update-nat=0 --publish"
#pjsua = "--null-audio --local-port 0 --rtp-port 0"

# Handle REGISTER first
req1 = sip.RecvfromTransaction("Initial REGISTER", 200,
				include=["REGISTER sip"], 
				exclude=[],
				resp_hdr=["Expires: 1800"]
			  	)

# First PUBLISH, reply with 412
req2 = sip.RecvfromTransaction("Initial PUBLISH, will be replied with 412", 412,
				include=["PUBLISH sip"], 
				exclude=["Expires:"]
			  	)

# Second PUBLISH
req3 = sip.RecvfromTransaction("Second PUBLISH, will be replied with 200", 200,
				include=["PUBLISH sip"], 
				exclude=["Expires:"],
				resp_hdr=["Expires: 60", "SIP-ETag: dx200xyz"]
			  	)

# PUBLISH refresh, respond with 408
req4 = sip.RecvfromTransaction("PUBLISH refresh, will be replied with 408", 408,
				include=["PUBLISH sip", "SIP-If-Match: dx200xyz"], 
				exclude=["Expires:"],
				resp_hdr=["Expires: 60", "SIP-ETag: dx200xyz"]
			  	)

# After 5 minutes, pjsua should retry again
req5 = sip.RecvfromTransaction("PUBLISH retry", 200,
				include=["PUBLISH sip"], 
				exclude=["Expires:", "SIP-If-Match:"],
				resp_hdr=["Expires: 60", "SIP-ETag: abc"]
			  	)



recvfrom_cfg = sip.RecvfromCfg("PUBLISH scenarios",
			       pjsua, [req1, req2, req3])




import inc_sip as sip
import inc_sdp as sdp

sdp = \
"""
v=0
o=- 0 0 IN IP4 127.0.0.1
s=pjmedia
c=IN IP4 127.0.0.1
t=0 0
m=audio 4000 RTP/AVP 0 101
a=rtpmap:0 PCMU/8000
a=sendrecv
a=rtpmap:101 telephone-event/8000
a=fmtp:101 0-15
"""

pjsua_args = "--null-audio --auto-answer 200 --timer-min-se 90"
extra_headers = "Supported: timer\nSession-Expires: 1800;refresher=uas\n"
include = ["Session-Expires:.*;refresher=ua[cs]"]
exclude = []
sendto_cfg = sip.SendtoCfg("Session Timer offer refresher uas", pjsua_args, sdp, 200, 
			   extra_headers=extra_headers,
			   resp_inc=include, resp_exc=exclude) 
			   



# $Id$
#
import inc_const as const

PJSUA = ["--null-audio --max-calls=1 --no-tcp $SIPP_URI"]

PJSUA_EXPECTS = [[0, const.STATE_EARLY, ""],
		 [0, "487/INVITE", ""],
		 [0, const.STATE_DISCONNECTED, ""]
		]

PJSUA_CLI_EXPECTS = [[0, const.STATE_EARLY, ""],
		     [0, "487/INVITE", ""],
		     [0, const.STATE_DISCONNECTED, ""]
		    ]



import sys
import inc_util as util

from inc_cfg import *

# Read configuration
cfg_file = util.load_module_from_file("cfg_file", ARGS[1])

# Here where it all comes together
test = cfg_file.test_param



import random
import config_site
import socket
import errno
import time

DEFAULT_ECHO = True
DEFAULT_TRACE = True
DEFAULT_START_SIP_PORT = 50000
DEFAULT_TELNET = True
DEFAULT_START_TELNET_PORT = 60000

# Shared vars
ARGS = []		# arguments containing script module & config
HAS_SND_DEV = config_site.HAS_SND_DEV

# Individual pjsua instance configuration class
class InstanceParam:
	# Name to identify this pjsua instance (e.g. "caller", "callee", etc.)
	name = ""
	# pjsua command line arguments, concatenated in string
	arg = ""
	# Specify whether pjsua telnet CLI is enabled
	telnet_enabled = DEFAULT_TELNET
	# Telnet port number, zero to automatically assign
	telnet_port = 0
	# Specify whether pjsua output should be echoed to stdout
	echo_enabled = DEFAULT_ECHO
	# Enable/disable test tracing
	trace_enabled = DEFAULT_TRACE
	# SIP URI to send request to this instance
	uri = ""
	# SIP port number, zero to automatically assign
	sip_port = 0
	# Does this have registration? If yes then the test function will
	# wait until the UA is registered before doing anything else
	have_reg = False
	# Does this have PUBLISH?
	have_publish = False
	# Enable stdout buffer?
	enable_buffer = False
	def __init__(	self, 
			name,			# Instance name
			arg, 			# Cmd-line arguments
			uri="", 		# URI
			uri_param="",		# Additional URI param
			telnet_port=0, 		# Telnet port
			sip_port=0, 		# SIP port
			have_reg=False,		# Have registration?
			have_publish=False,	# Have publish?
			echo_enabled=DEFAULT_ECHO, 
			trace_enabled=DEFAULT_TRACE,
			telnet_enabled = DEFAULT_TELNET,
			enable_buffer = False):
		# Instance name
		self.name = name
		# Give random telnet_port if it's not specified and telnet CLI is enabled
		if telnet_enabled and telnet_port==0:
			# avoid port conflict
			cnt = 0
			port = 0
			while cnt < 10:
				cnt = cnt + 1
				port = random.randint(DEFAULT_START_TELNET_PORT, 65534)
				s = socket.socket(socket.AF_INET)
				try:
					s.bind(("0.0.0.0", port))
				except socket.error as serr:
					s.close()
					if serr.errno ==  errno.EADDRINUSE or serr.errno == errno.EACCES:
						continue
				s.close()
				break;
			self.telnet_port = port
		else:
			self.telnet_port = telnet_port
		# Give random sip_port if it's not specified
		if sip_port==0:
			# avoid port conflict
			cnt = 0
			port = 0
			while cnt < 10:
				port = random.randint(DEFAULT_START_SIP_PORT, 60000)
				if port==self.telnet_port:
					continue
				cnt = cnt + 1
				s = socket.socket(socket.AF_INET)
				try:
					s.bind(("0.0.0.0", port))
				except socket.error as serr:
					s.close()
					if serr.errno ==  errno.EADDRINUSE or serr.errno ==  errno.EACCES:
						continue
				s.close()
				break;
			self.sip_port = port
			# Give some time for socket close
			time.sleep(0.5)
		else:
			self.sip_port = sip_port
		# Autogenerate URI if it's empty.
		self.uri = uri
		if self.uri=="":
			self.uri = "sip:pjsip@127.0.0.1:" + str(self.sip_port)
		# Add uri_param to the URI
		self.uri = self.uri + uri_param
		# Add bracket to the URI
		if self.uri[0] != "<":
			self.uri = "<" + self.uri + ">"
		# Add SIP local port to the argument
		self.arg = arg + " --local-port=" + str(self.sip_port)
		self.have_reg = have_reg
		self.have_publish = have_publish
		if have_publish and have_reg and not ("--publish" in self.arg):
			self.arg = self.arg + " --publish"
		self.echo_enabled = echo_enabled
		self.trace_enabled = trace_enabled
		self.enable_buffer = enable_buffer


############################################
# Test parameter class
class TestParam:
	title = ""
	# params is list containing InstanceParams objects
	inst_params = []
	# flag if this tes should be skipped
	skip = None
	# list of Expect instances, to be filled at run-time by
	# the test program	
	process = []
	# the function for test body
	test_func = None
	post_func = None
	def __init__(	self, 
			title, 		# Test title
			inst_params, 	# InstanceParam's as list
			func=None,
			skip=False,
			post_func=None,
			need_stdout_buffer=False):
		self.title = title
		self.inst_params = inst_params
		self.skip = skip
		self.test_func = func
		self.post_func = post_func


###################################
# TestError exception
class TestError(Exception):
	desc = ""
	def __init__(self, desc):
		self.desc = desc





#
# pjsua Python GUI Demo
#
# Copyright (C)2013 Teluu Inc. (http://www.teluu.com)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
#
import sys
if sys.version_info[0] >= 3: # Python 3
    import tkinter as tk
    from tkinter import ttk
    from tkinter import messagebox as msgbox
else:
    import Tkinter as tk
    import tkMessageBox as msgbox
    import ttk

import random
import pjsua2 as pj
import _pjsua2
import accountsetting
import application
import call
import chat as ch

write=sys.stdout.write

# Account class
class Account(pj.Account):
    """
    High level Python Account object, derived from pjsua2's Account object.
    """
    def __init__(self, app):
        pj.Account.__init__(self)
        self.app = app
        self.randId = random.randint(1, 9999)
        self.cfg =  pj.AccountConfig()
        self.cfgChanged = False
        self.buddyList = []
        self.chatList = []
        self.deleting = False

    def findChat(self, uri_str):
        uri = ch.ParseSipUri(uri_str)
        if not uri: return None

        for chat in self.chatList:
            if chat.isUriParticipant(uri) and chat.isPrivate():
                return chat
        return None

    def newChat(self, uri_str):
        uri = ch.ParseSipUri(uri_str)
        if not uri: return None

        chat = ch.Chat(self.app, self, uri)
        self.chatList.append(chat)
        self.app.updateWindowMenu()
        return chat

    def statusText(self):
        status = '?'
        if self.isValid():
            ai = self.getInfo()
            if ai.regLastErr:
                status = self.app.ep.utilStrError(ai.regLastErr)
            elif ai.regIsActive:
                if ai.onlineStatus:
                    if len(ai.onlineStatusText):
                        status = ai.onlineStatusText
                    else:
                        status = "Online"
                else:
                    status = "Registered"
            else:
                if ai.regIsConfigured:
                    if ai.regStatus/100 == 2:
                        status = "Unregistered"
                    else:
                        status = ai.regStatusText
                else:
                    status = "Doesn't register"
        else:
            status = '- not created -'
        return status

    def onRegState(self, prm):
        self.app.updateAccount(self)

    def onIncomingCall(self, prm):
        c = call.Call(self, call_id=prm.callId)
        call_prm = pj.CallOpParam()
        call_prm.statusCode = 180
        c.answer(call_prm)
        ci = c.getInfo()
        msg = "Incoming call for account '%s'" % self.cfg.idUri
        if msgbox.askquestion(msg, "Accept call from '%s'?" % (ci.remoteUri), default=msgbox.YES) == u'yes':
            call_prm.statusCode = 200
            c.answer(call_prm)

            # find/create chat instance
            chat = self.findChat(ci.remoteUri)
            if not chat: chat = self.newChat(ci.remoteUri)

            chat.showWindow()
            chat.registerCall(ci.remoteUri, c)
            chat.updateCallState(c, ci)
        else:
            c.hangup(call_prm)

    def onInstantMessage(self, prm):
        chat = self.findChat(prm.fromUri)
        if not chat: chat = self.newChat(prm.fromUri)

        chat.showWindow()
        chat.addMessage(prm.fromUri, prm.msgBody)

    def onInstantMessageStatus(self, prm):
        if prm.code/100 == 2: return

        chat = self.findChat(prm.toUri)
        if not chat:
            write("=== IM status to " + prm.toUri + "cannot find chat\r\n")
            return

        chat.addMessage(None, "Failed sending message to '%s': %s" % (prm.toUri, prm.reason))

    def onTypingIndication(self, prm):
        chat = self.findChat(prm.fromUri)
        if not chat:
            write("=== Incoming typing indication from " + prm.fromUri + "cannot find chat\r\n")
            return

        chat.setTypingIndication(prm.fromUri, prm.isTyping)


# Account frame, to list accounts
class AccountListFrame(ttk.Frame):
    """
    This implements a Frame which contains account list and buttons to operate
    on them (Add, Modify, Delete, etc.).
    """
    def __init__(self, parent, app, acc_list = []):
        ttk.Frame.__init__(self, parent, name='acclist')
        self.app = app
        self.accList = acc_list
        self.accDeletedList = []
        self.pack(expand='yes', fill='both')
        self._createWidgets()
        for acc in self.accList:
            self._showAcc(acc)

    def _createWidgets(self):
        self.tv = ttk.Treeview(self, columns=('ID', 'Registrar', 'Default'), selectmode='browse')
        self.tv.heading('#0', text='Priority')
        self.tv.heading(0, text='ID')
        self.tv.heading(1, text='Registrar')
        self.tv.heading(2, text='Default?')
        self.tv.column('#0', width=60)
        self.tv.column(0, width=300)
        self.tv.column(1, width=200)
        self.tv.column(2, width=60)
        self.tv.grid(column=0, row=0, rowspan=4, padx=5, pady=5)

        ttk.Button(self, text='Add..', command=self._onBtnAdd).grid(column=1, row=0, padx=5)
        ttk.Button(self, text='Settings..', command=self._onBtnSettings).grid(column=1, row=1)
        ttk.Button(self, text='Set Default', command=self._onBtnSetDefault).grid(column=1, row=2)
        ttk.Button(self, text='Delete..', command=self._onBtnDelete).grid(column=1, row=3)

    def _showAcc(self, acc):
        is_default = 'Yes' if acc.isValid() and acc.isDefault() else ''
        values = (acc.cfg.idUri, acc.cfg.regConfig.registrarUri, is_default)
        self.tv.insert('', 0, str(acc.randId), open=True, text=str(acc.cfg.priority), values=values)

    def updateAccount(self, acc):
        is_default = 'Yes' if acc.isValid() and acc.isDefault() else ''
        values = (acc.cfg.idUri, acc.cfg.regConfig.registrarUri, is_default)
        self.tv.item(str(acc.randId), text=str(acc.cfg.priority), values=values)

    def _getSelectedAcc(self):
        items = self.tv.selection()
        if not items:
            return None
        iid = int(items[0])
        return [acc for acc in self.accList if acc.randId==iid][0]

    def _onBtnAdd(self):
        cfg = pj.AccountConfig()
        dlg = accountsetting.Dialog(self.master, cfg)
        if dlg.doModal():
            acc = Account(self.app)
            acc.cfg = cfg
            self._showAcc(acc)
            self.accList.append(acc)
            self.cfgChanged = True

    def _onBtnSettings(self):
        acc = self._getSelectedAcc()
        if not acc:
            return
        dlg = accountsetting.Dialog(self.master, acc.cfg)
        if dlg.doModal():
            self.updateAccount(acc)
            self.cfgChanged = True

    def _onBtnDelete(self):
        acc = self._getSelectedAcc()
        if not acc:
            return
        msg = "Do you really want to delete account '%s'" % acc.cfg.idUri
        if msgbox.askquestion('Delete account?', msg, default=msgbox.NO) != u'yes':
            return
        self.accList.remove(acc)
        self.accDeletedList.append(acc)
        self.tv.delete( (str(acc.randId),) )

    def _onBtnSetDefault(self):
        acc = self._getSelectedAcc()
        if not acc:
            return
        if acc.isValid():
            acc.setDefault()
        for acc in self.accList:
            self.updateAccount(acc)


if __name__ == '__main__':
    application.main()



#
import inc_const as const

PJSUA = ["--null-audio --extra-audio --max-calls=1 --no-tcp $SIPP_URI"]

# Send hold after remote holds (double hold)
PJSUA_EXPECTS = [[0, const.MEDIA_HOLD, ""],
		 [0, "ACK sip:", "H"]
		]



#
# cfg_symbian.py - Symbian target configurator
#
# Copyright (C) 2008-2009 Teluu Inc. (http://www.teluu.com)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
import builder
import os
import sys

# Each configurator must export this function
def create_builder(args):
    usage = """\
Usage:
  main.py cfg_symbian [-h|--help] [-t|--target TARGET] [cfg_site]
 
Arguments:
  cfg_site:            site configuration module. If not specified, "cfg_site" 
                       is implied
  -t,--target TARGET:  Symbian target to build. Default is "gcce urel". 
                       Other values:
                        "winscw udeb", "gcce udeb", etc.
  -h, --help           Show this help screen
"""           

    cfg_site = "cfg_site"
    target = "gcce urel"
    in_option = ""
    
    for arg in args:
        if in_option=="-t":
            target = arg
            in_option = ""
        elif arg=="--target" or arg=="-t":
            in_option = "-t"
        elif arg=="--help" or arg=="-h":
            print usage
            sys.exit(0)
        elif arg[0]=="-":
            print usage
            sys.exit(1)
        else:
            cfg_site = arg
        
    if os.access(cfg_site+".py", os.F_OK) == False:
        print "Error: file '%s.py' doesn't exist." % (cfg_site)
        sys.exit(1)

    cfg_site = __import__(cfg_site)
    test_cfg = builder.BaseConfig(cfg_site.BASE_DIR, \
                                  cfg_site.URL, \
                                  cfg_site.SITE_NAME, \
                                  cfg_site.GROUP, \
                                  cfg_site.OPTIONS)
    config_site1 = """\
#define PJ_TODO(x)
#include <pj/config_site_sample.h>

"""

    config_Site = config_site1 + cfg_site.CONFIG_SITE

    builders = [
        builder.SymbianTestBuilder(test_cfg, 
                                   target=target,
                                   build_config_name="default",
                                   config_site=config_site1,
                                   exclude=cfg_site.EXCLUDE,
                                   not_exclude=cfg_site.NOT_EXCLUDE)
        ]

    return builders




import inc_sip as sip
import inc_sdp as sdp

# Session timers retry after 422


pjsua = "--null-audio sip:127.0.0.1:$PORT --timer-min-se 100 --timer-se 1000 --no-tcp"

# First INVITE with timer rejected with 422
req1 = sip.RecvfromTransaction("INVITE with SE too small", 422,
				include=["Session-Expires:\s*1000"], 
				exclude=[],
				resp_hdr=["Min-SE: 2000"]
			  	)

# Wait for ACK
req2 = sip.RecvfromTransaction("Wait ACK", 0, include=["ACK sip"])

# New INVITE with SE >= Min-SE
req3 = sip.RecvfromTransaction("Retrying with acceptable SE", 200,
				include=["Session-Expires:\s*2000", "Min-SE:\s*2000"], 
				exclude=[],
				resp_hdr=["Session-Expires: 2000;refresher=uac"]
			  	)


recvfrom_cfg = sip.RecvfromCfg("Session timers retry after 422",
			       pjsua, [req1, req2, req3])




import inc_sip as sip
import inc_sdp as sdp

pjsua = "--null-audio --id=sip:CLIENT --registrar sip:127.0.0.1:$PORT " + \
	"--username theuser1 --realm python1 --password passwd --next-cred " + \
	"--username theuser2 --realm python2 --password passwd " + \
	"--auto-update-nat=0"

req1 = sip.RecvfromTransaction("Initial registration", 401,
				include=["REGISTER sip"], 
				resp_hdr=["WWW-Authenticate: Digest realm=\"python1\", nonce=\"1234\"",
					  "WWW-Authenticate: Digest realm=\"python2\", nonce=\"6789\""],
				expect="SIP/2.0 401"
			  )

req2 = sip.RecvfromTransaction("Registration retry with auth (not allowed multiple auth)", 200,
				include=["REGISTER sip", 
					 "Authorization:", # [\\s\\S]+Authorization:"
					 "realm=\"python1\"", # "realm=\"python2\"", 
					 "username=\"theuser1\"", # "username=\"theuser2\"", 
					 "nonce=\"1234\"", # "nonce=\"6789\"", 
					 "response="],
				expect="registration success",
				pj_config="PJSIP_AUTH_ALLOW_MULTIPLE_AUTH_HEADER.*: 0"
			  )

req3 = sip.RecvfromTransaction("Registration retry with auth (allowed multiple auth)", 200,
				include=["REGISTER sip", 
					 "Authorization:[\\s\\S]+Authorization:", # Must have 2 Auth hdrs
					 "realm=\"python1\"", "realm=\"python2\"", 
					 "username=\"theuser1\"", "username=\"theuser2\"", 
					 "nonce=\"1234\"", "nonce=\"6789\"", 
					 "response="],
				expect="registration success",
				pj_config="PJSIP_AUTH_ALLOW_MULTIPLE_AUTH_HEADER.*: 1"	     
			  )                          

recvfrom_cfg = sip.RecvfromCfg("Multiple authentication challenges",
			       pjsua, [req1, req2, req3], pj_config="PJSIP_AUTH_ALLOW_MULTIPLE_AUTH_HEADER")
